﻿namespace HeuristicLab.Algorithms.PushGP.Expressions
{
  using HeuristicLab.Algorithms.PushGP.Interpreter;
  using HeuristicLab.Algorithms.PushGP.Stack;

  /// <summary>
  ///     Pushes TRUE onto the BOOLEAN stack if the top two items are equal, or FALSE otherwise
  /// </summary>
  /// <typeparam name="T">Stacktype</typeparam>
  public abstract class EqualsExpression<T> : StatelessExpression
  {
    public void Eval(IStack<T> stack, IStack<bool> booleanStack)
    {
      if (stack.Count < 2) return;

      var items = stack.Pop(2);

      booleanStack.Push(items[0].Equals(items[1]));
    }
  }

  public class IntegerEqualsExpression : EqualsExpression<long>
  {
    protected override string InitStringRepresentation()
    {
      return "INTEGER.=";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.IntegerStack, interpreter.BooleanStack);
    }
  }

  public class FloatEqualsExpression : EqualsExpression<double>
  {
    protected override string InitStringRepresentation()
    {
      return "FLOAT.=";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.FloatStack, interpreter.BooleanStack);
    }
  }

  public class BooleanEqualsExpression : EqualsExpression<bool>
  {
    protected override string InitStringRepresentation()
    {
      return "BOOLEAN.=";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.BooleanStack, interpreter.BooleanStack);
    }
  }

  public class NameEqualsExpression : EqualsExpression<string>
  {
    protected override string InitStringRepresentation()
    {
      return "NAME.=";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.NameStack, interpreter.BooleanStack);
    }
  }

  public class ExecEqualsExpression : EqualsExpression<Expression>
  {
    protected override string InitStringRepresentation()
    {
      return "EXEC.=";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.ExecStack, interpreter.BooleanStack);
    }
  }

  public class CodeEqualsExpression : EqualsExpression<Expression>
  {
    protected override string InitStringRepresentation()
    {
      return "CODE.=";
    }

    public override void Eval(IPushGpInterpreter interpreter)
    {
      this.Eval(interpreter.CodeStack, interpreter.BooleanStack);
    }
  }
}