﻿namespace HeuristicLab.Algorithms.PushGP.Expressions {
  using HeuristicLab.Algorithms.PushGP.Interpreter;
  using HeuristicLab.Algorithms.PushGP.Stack;

  public abstract class PushExpression<T> : StatefullExpression<T> {

    protected PushExpression(T state) : base(state) { }

    protected override int InitId() {
      var hash = 19;

      hash = hash * 31 + this.GetType().GetHashCode();
      hash = hash * 31 + this.State.GetHashCode();

      return hash;
    }

    protected void Eval(IStack<T> stack) {
      stack.Push(this.State);
    }

    public override bool Equals(object obj) {
      if (!(obj is PushExpression<T>)) return false;
      var other = (PushExpression<T>)obj;

      return this.State.Equals(other.State);
    }

    public override int GetHashCode() {
      return base.GetHashCode();
    }
  }

  public class IntegerPushExpression : PushExpression<long> {
    public IntegerPushExpression(long state)
      : base(state) {
    }

    protected override string InitStringRepresentation() {
      return "IntegerPush";
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.IntegerStack);
    }
  }

  public class FloatPushExpression : PushExpression<double> {
    public FloatPushExpression(double state)
      : base(state) {
    }

    protected override string InitStringRepresentation() {
      return "FloatPush";
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.FloatStack);
    }
  }

  public class BooleanPushExpression : PushExpression<bool> {
    public BooleanPushExpression(bool state)
      : base(state) {
    }

    protected override string InitStringRepresentation() {
      return "BooleanPush";
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.BooleanStack);
    }
  }

  public class NamePushExpression : PushExpression<string> {
    public NamePushExpression(string state)
      : base(state) {
    }

    protected override string InitStringRepresentation() {
      return "NamePush";
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.NameStack);
    }
  }

  public class ExecPushExpression : PushExpression<Expression> {
    public ExecPushExpression(Expression state)
      : base(state) {
    }

    protected override string InitStringRepresentation() {
      return "ExecPush";
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.ExecStack);
    }
  }
}